﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Windows.UI.Xaml;

namespace G.Controls
{
    public class ControlService
    {
        #region 控件显示或隐藏，主要是配合绑定来做
        /// <summary>
        /// 通过绑定某个内容来判断是否显示或者隐藏控件，如果内容为空那么显示否则隐藏，内容如果是字符串，那么也可以有类似的判断， 主要是用来做类似watemarket效果
        /// </summary>
        public static readonly DependencyProperty IsNullToVisibleProperty = DependencyProperty.RegisterAttached("IsNullToVisible", typeof(object), typeof(ControlService), new PropertyMetadata(null, OnIsNullToVisibleChanged));

        private static void OnIsNullToVisibleChanged(DependencyObject o, DependencyPropertyChangedEventArgs e)
        {
            UIElement u = o as UIElement;
            object obj = ControlService.GetIsNullToVisible(u);
            if (obj == null)
            {
                u.Visibility = Visibility.Visible;
            }
            else
            {
                if (obj is string)
                {
                    if (string.IsNullOrEmpty(obj as string))
                    {
                        u.Visibility = Visibility.Visible;
                    }else
                        u.Visibility = Visibility.Collapsed;
                }
                else
                    u.Visibility = Visibility.Collapsed;
            }
        }

        public static object GetIsNullToVisible(UIElement element)
        {
            if (element == null)
            {
                throw new ArgumentNullException("element");
            }
            return element.GetValue(IsNullToVisibleProperty);
        }

        public static void SetIsNullToVisible(UIElement element, object value)
        {
            if (element == null)
            {
                throw new ArgumentNullException("element");
            }
            element.SetValue(IsNullToVisibleProperty, value);
        }
        #endregion

        #region AttachProperty
        /// <summary>
        /// 对某个元素附加属性，属性类型为object 类型，可以配合Datatemplate来实现一个附加的绑定模板
        /// </summary>
        public static readonly DependencyProperty AttachPropertyProperty = DependencyProperty.RegisterAttached("AttachProperty", typeof(object), typeof(ControlService), null);

        public static object GetAttachProperty(DependencyObject obj)
        {
            return obj.GetValue(AttachPropertyProperty);
        }

        public static void SetAttachProperty(DependencyObject obj, object value)
        {
            obj.SetValue(AttachPropertyProperty, value);
        }

        /// <summary>
        /// 对摸个元素附加属性的模板
        /// </summary>
        public static readonly DependencyProperty AttachPropertyTemplateProperty = DependencyProperty.RegisterAttached("AttachPropertyTemplate", typeof(DataTemplate), typeof(ControlService), null);

        public static DataTemplate GetAttachPropertyTemplate(DependencyObject obj)
        {
            return obj.GetValue(AttachPropertyTemplateProperty) as DataTemplate;
        }

        public static void SetAttachPropertyTemplate(DependencyObject obj, DataTemplate value)
        {
            obj.SetValue(AttachPropertyTemplateProperty, value);
        }
        #endregion

    }
}
