﻿using System;
using System.Collections.Generic;
using System.Linq;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Documents;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;

// The Templated Control item template is documented at http://go.microsoft.com/fwlink/?LinkId=234235

namespace G.Controls
{
    [TemplateVisualState(GroupName = GroupStateName, Name = NormalState)]
    [TemplateVisualState(GroupName = GroupStateName, Name = LoadedState)]
    public class LoadingImageControl : ContentControl
    {
        const string GroupStateName = "LoadingState";
        const string NormalState = "Normal";
        const string LoadedState = "Loaded";

        public static readonly DependencyProperty ImageSourceProperty = DependencyProperty.Register("ImageSource", typeof(ImageSource), typeof(LoadingImageControl), new PropertyMetadata(null, ImageSourcePropertyChanged));

        static void ImageSourcePropertyChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            LoadingImageControl control = d as LoadingImageControl;
            if (control != null)
            {
                VisualStateManager.GoToState(control, NormalState, false);
                Windows.UI.Xaml.Media.Imaging.BitmapImage image = e.NewValue as Windows.UI.Xaml.Media.Imaging.BitmapImage;
                if (image != null)
                    image.ImageOpened += control.ImageOpened;
            }
        }



        private void ImageOpened(object sender, RoutedEventArgs e)
        {
            VisualStateManager.GoToState(this, LoadedState, false);
        }

        public ImageSource ImageSource
        {
            get
            {
                return this.GetValue(ImageSourceProperty) as ImageSource;
            }
            set
            {
                this.SetValue(ImageSourceProperty, value);
            }
        }



        public LoadingImageControl()
        {
            this.DefaultStyleKey = typeof(LoadingImageControl);
        }
    }
}
