﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Input;


namespace G.Net
{
    public class HttpGet : HttpRequest
    {
        private ParamCollection queryParams = new ParamCollection();

        /// <summary>
        /// Initializes a new instance of <see cref="HttpGet"/> with the specified <paramref name="uri"/>.
        /// </summary>
        /// <param name="uri">The uri to identify a resource in the remote server.</param>
        public HttpGet(string uri)
            : base(uri)
        {
            base.Method = System.Net.Http.HttpMethod.Get;
        }

        /// <summary>
        /// Gets or sets parameters to be sent in the request query string.
        /// </summary>
        public virtual ParamCollection Params
        {
            get
            {
                return queryParams;
            }
        }

        /// <summary>
        /// Constructs the uri by appending the query parameters in <see cref="Params"/>.
        /// </summary>
        /// <returns>The uri with query string appended.</returns>
        protected override string ConstructUri()
        {
            var uri = Uri;
            if (null != Params)
            {
                uri += "?";
                foreach (var item in Params)
                {
                    uri += string.Format("{0}={1}&", item.Name, System.Uri.EscapeDataString(item.Value));
                }
                uri = uri.TrimEnd('&');
            }
            return uri;
        }
    }
}
